package net.mc_pandacraft.java.plugin.pandacraftutils.commands;

import java.util.ArrayList;
import java.util.List;

import net.mc_pandacraft.java.plugin.pandacraftutils.modules.protection.TamedEntityProtectManager;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OffPlayer;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OnlinePlayer;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OnlinePlayerManager;
import net.mc_pandacraft.java.util.StringUtil;
import net.minecraft.server.v1_7_R1.EntityHorse;
import net.minecraft.server.v1_7_R1.GenericAttributes;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.craftbukkit.v1_7_R1.entity.CraftHorse;
import org.bukkit.entity.Animals;
import org.bukkit.entity.Damageable;
import org.bukkit.entity.Horse;
import org.bukkit.entity.Player;
import org.bukkit.entity.Tameable;

import com.google.common.collect.ImmutableList;

public class CommandAnimal extends AbstractCommandExecutor {
	
	public CommandAnimal() {
		super("animal");
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		
		if (!(sender instanceof Player)) {
			sender.sendMessage(ChatColor.RED+"seul un joueur en ligne peut effectuer cette commande");
			return true;
		}
		
		Player p = (Player) sender;
		OnlinePlayer op = OnlinePlayerManager.get(p);
		
		if (!op.isAnimalSelectionEnabled()) {
			op.enableAnimalSelection();
			p.sendMessage(ChatColor.GREEN+"Faites un clic droite pour sélectionner un animal apprivoisé (cheval, loup ou chat)");
			return true;
		}
		// la sélection d'un animal est déjà activée
		
		
		// si le joueur veut désactiver la sélection d'un animal
		if (args.length == 0 || args[0].equalsIgnoreCase("annuler")) {
			p.sendMessage(ChatColor.GREEN+"La sélection d'un animal est désactivée");
			op.resetAnimalSelection();
			return true;
		}
		
		
		// si je joueur n'a pas encore sélectionné d'animal
		if (op.getSelectedAnimal() == null) {
			p.sendMessage(ChatColor.RED+"Faites d'abord un clic droit sur un animal apprivoisé (cheval, loup ou chat)");
			p.sendMessage(ChatColor.RED+"Pour désactiver la sélection, faites "+ChatColor.GRAY+"/animal annuler");
			return true;
		}

		TamedEntityProtectManager tepm = plugin.tamedEntityProtectManager;
		
		Tameable tameable = op.getSelectedAnimal();
		String animalName = tepm.get_thisAnimal_Name(tameable);
		
		if (!op.hasPermission("pandacraft.animal.admin")) {
			if (tameable.getOwner() == null) {
				p.sendMessage(ChatColor.RED+animalName+ChatColor.RED+" n'appartient à personne");
				return true;
			}
			if (!tameable.getOwner().equals(p)) {
				p.sendMessage(ChatColor.RED+animalName+ChatColor.RED+" n'est pas à vous");
				return true;
			}
		}
		
		
		
		
		if (args.length >= 2 && args[0].equalsIgnoreCase("donner")) {
			
			Player newOwner = plugin.getServer().getPlayer(args[1]);
			
			if (newOwner == null) {
				p.sendMessage(ChatColor.RED+"Le joueur indiqué n'est pas en ligne");
				return true;
			}
			
			tameable.setOwner(newOwner);
			p.sendMessage(ChatColor.GREEN+"Vous venez de donner "+animalName+ChatColor.GREEN+" à "+newOwner.getDisplayName());
			op.resetAnimalSelection();
			
			return true;
		}
		
		if (args.length >= 2 && args[0].equalsIgnoreCase("modifier")) {
			
			if (!op.hasPermission("pandacraft.animal.modify")) {
				p.sendMessage(ChatColor.RED+"Vous n'avez pas la permission de modifier les attributs de l'animal");
				return true;
			}
		
			if (args.length >= 3 && args[1].equalsIgnoreCase("hauteur_saut")) {
			
				if (!(tameable instanceof Horse)) {
					p.sendMessage(ChatColor.RED+"Vous ne pouvez pas modifier la hauteur de saut de cet animal");
					return true;
				}
				Horse h = (Horse)tameable;
				
				try {
					double jump = Double.parseDouble(args[2]);
					if (jump < 0) throw new NumberFormatException();
					p.sendMessage(ChatColor.GREEN+"Puissance de saut du cheval défini à "+StringUtil.formatDouble(jump));
					jump *= 0.2D;
					h.setJumpStrength(jump);
				} catch (NumberFormatException e) {
					p.sendMessage(ChatColor.RED+"La valeur indiquée est invalide. Nombre entre 0 et 10 (nombre décimal possible)");
				}
				return true;
			}
			if (args.length >= 3 && args[1].equalsIgnoreCase("vitesse")) {

				if (!(tameable instanceof Horse)) {
					p.sendMessage(ChatColor.RED+"Vous ne pouvez pas modifier la vitesse de cet animal");
					return true;
				}
				
				Horse h = (Horse)tameable;
				EntityHorse horse = ((CraftHorse)h).getHandle();
				
				try {
					double speed = Double.valueOf(args[2]).doubleValue();
					if (speed < 0 || speed > 100) throw new NumberFormatException();
					p.sendMessage(ChatColor.GREEN+"Vitesse du cheval défini à "+StringUtil.formatDouble(speed));
					speed *= 0.1D;
					horse.getAttributeInstance(GenericAttributes.d).setValue(speed);
				} catch (NumberFormatException e) {
					p.sendMessage(ChatColor.RED+"La valeur indiquée est invalide. Nombre entre 0 et 100 (nombre décimal possible)");
				}
				return true;
			}
		}

		if (args.length >= 1 && args[0].equalsIgnoreCase("info")) {
			if (!(tameable instanceof Animals)) {
				p.sendMessage(ChatColor.RED+"L'entité sélectionné n'est pas un animal (erreur innatendu)");
				return true;
			}
			Animals animal = (Animals) tameable;
			p.sendMessage(ChatColor.GOLD+"----- Information sur l'animal sélectionné -----");
			
			if (tameable.getOwner() != null)
				p.sendMessage(ChatColor.GOLD+"Propriétaire : "+ChatColor.RESET+(new OffPlayer(tameable.getOwner().getName()).getDisplayName()));
			else
				p.sendMessage(ChatColor.GOLD+"Propriétaire : "+ChatColor.GRAY+"Non apprivoisé");
			p.sendMessage(ChatColor.GOLD+"Nom personnalisé : "+ChatColor.RESET+((animal.getCustomName() != null) ? animal.getCustomName() : ChatColor.GRAY+"Aucun"));
			p.sendMessage(ChatColor.GOLD+"Point de vie : "+ChatColor.GRAY+((int)((Damageable)animal).getHealth())+"/"+((int)((Damageable)animal).getMaxHealth())+ChatColor.GOLD+" (2 points = 1 "+ChatColor.RED+"❤"+ChatColor.GOLD+")");
			p.sendMessage(ChatColor.GOLD+"Respiration sous l'eau : "+ChatColor.GRAY+animal.getRemainingAir()+"/"+animal.getMaximumAir());
			
			if (animal instanceof Horse) {
				Horse cheval = (Horse) animal;
				
				p.sendMessage(ChatColor.GOLD+"Niveau de domestication : "+ChatColor.GRAY+cheval.getDomestication()+"/"+cheval.getMaxDomestication());
				p.sendMessage(ChatColor.GOLD+"Puissance de saut : "+ChatColor.GRAY+(cheval.getJumpStrength()/0.2D));

				double speed = ((CraftHorse)cheval).getHandle().getAttributeInstance(GenericAttributes.d).getValue();
				p.sendMessage(ChatColor.GOLD+"Vitesse de course : "+ChatColor.GRAY+(speed/0.1D));
			}
			
			return true;
		}
		
		
		
		
		return false;
	}
	
	
	

	public static final List<String> SECOND_ARG_MODIFIER = ImmutableList.of("hauteur_saut", "vitesse");
	
	
	@Override
	public List<String> onTabComplete(CommandSender sender, Command cmd, String alias, String[] args) {
		
		
		if (!(sender instanceof Player)) {
			return NO_PROPOSAL;
		}
		
		Player p = (Player) sender;
		OnlinePlayer op = OnlinePlayerManager.get(p);
		
		if (!op.isAnimalSelectionEnabled()) {
			return NO_PROPOSAL;
		}
		// la sélection d'un animal est déjà activée
		List<String> proposal_first_arg = new ArrayList<String>();
		
		proposal_first_arg.add("annuler");
		

		if (op.getSelectedAnimal() != null) {
			proposal_first_arg.add("donner");
			proposal_first_arg.add("modifier");
			proposal_first_arg.add("info");
		}
		
		
		
		
		if (args.length == 1) {
			return getTabProposalFromToken(args[0], proposal_first_arg);
		}
		else if (args.length > 1 && op.getSelectedAnimal() != null) {
				if (args[0].equalsIgnoreCase("donner"))
					if (args.length == 2)
						return null;
				if (args[0].equalsIgnoreCase("modifier")) {
					if (args.length == 2)
						return getTabProposalFromToken(args[1], SECOND_ARG_MODIFIER);
				}
		}
		
		
		
		return NO_PROPOSAL;
	}

}
