package net.mc_pandacraft.java.plugin.pandacraftutils.commands;

import java.text.DateFormat;
import java.util.List;
import java.util.Locale;

import mkremins.fanciful.FancyMessage;
import net.mc_pandacraft.java.plugin.pandacraftutils.modules.PrivateMessagesManager.MessageSender;
import net.mc_pandacraft.java.plugin.pandacraftutils.modules.PrivateMessagesManager.MessageSendingException;
import net.mc_pandacraft.java.plugin.pandacraftutils.modules.PrivateMessagesManager.PrivateDiscussion;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OnlinePlayerManager;

import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;

public class CommandTell extends AbstractCommandExecutor {
	
	public CommandTell() {
		super("tell");
	}

	@Override
	public boolean onCommand(CommandSender sender, Command cmd, String label,
			String[] args) {
		
		if (args.length == 0 && sender instanceof Player) {
			Player player = (Player) sender;
			onCommandDiscussionsList(player, 5);
			return true;
		}
		
		if (args.length == 0) {
			sender.sendMessage(ChatColor.RED+"Seul un joueur en ligne peut voir la liste de ses discussions.");
			return true;
		}
		
		if (args.length >= 1 && args[0].equalsIgnoreCase("?")) {
			showHelp(sender);
			return true;
		}
		
		if (args.length >= 1 && args[0].toLowerCase().startsWith("d:") && sender instanceof Player) {
			Player player = (Player) sender;
			int nb = 5;
			try {
				if (args.length >= 2)
				nb = Integer.parseUnsignedInt(args[1]);
			} catch(NumberFormatException e) { }
			onCommandDiscussionDisplay(player, args[0].substring(2), nb);
			return true;
		}
		
		if (args.length >= 1 && args[0].toLowerCase().startsWith("l:") && sender instanceof Player) {
			Player player = (Player) sender;
			try {
				int nb = Integer.parseUnsignedInt(args[0].substring(2));
				
				onCommandDiscussionsList(player, nb);
				
			} catch(NumberFormatException e) {
				sender.sendMessage(ChatColor.RED+"Vous devez mettre un nombre entier");
			}
			return true;
		}
		
		if (args.length >= 2) {
			onCommandSend(sender, args[0], getLastParam(args, 1));
			return true;
		}
		
		
		
		
		sender.sendMessage(ChatColor.RED+"Mauvaise utilisation de la commande. Faites "+ChatColor.GRAY+"/m ?"+ChatColor.RED+" pour afficher l'aide");
		return true;
	}
	
	
	
	
	
	private void showHelp(CommandSender sender) {
		if (sender instanceof Player) {
			String[] m = new String[] {
					ChatColor.GOLD+"--------------- Messagerie privée ---------------",
					ChatColor.GRAY+"/m <Pseudo> <Message>"+ChatColor.GOLD+" pour envoyer un message à un joueur même quand il est hors-ligne",
					ChatColor.GRAY+"/m g:<NomGroupe> <Message>"+ChatColor.GOLD+" pour envoyer un message dans une discussion avec plusieurs joueurs",
					ChatColor.GRAY+"/m"+ChatColor.GOLD+" pour voir vos dernières conversations et lire les messages non lus",
					ChatColor.GRAY+"/m l:<Nombre>"+ChatColor.GOLD+" pour voir les dernières conversations. Le nombre correspond au nombre de conversation affiché",
					ChatColor.GRAY+"/m d:<Pseudo> [Nombre]"+ChatColor.GOLD+" pour voir les derniers messages avec ce joueur. Le nombre correspond au nombre de message à afficher",
					ChatColor.GRAY+"/m d:g:<NomGroupe> [Nombre]"+ChatColor.GOLD+" pour voir les derniers messages de ce groupe",
					ChatColor.GRAY+"/r <Message>"+ChatColor.GOLD+" pour répondre à un joueur (ou au groupe) à qui vous avez parlé la dernière fois",
					ChatColor.GRAY+"/r"+ChatColor.GOLD+" pour afficher le joueur (ou le groupe) à qui vous avez parlé la dernière fois",
					ChatColor.GOLD+"----------------------------------------------",
			};
			
			sender.sendMessage(m);
		}
		else {
			String[] m = new String[] {
					ChatColor.GOLD+"--------------- Messagerie privée ---------------",
					ChatColor.GOLD+"Information : Vous apparaitrez en tant que \"Système\" pour les destinataires de vos messages, et ils ne pourront pas vous répondre.",
					ChatColor.GRAY+"/m <Pseudo> <Message>"+ChatColor.GOLD+" pour envoyer un message à un joueur même quand il est hors-ligne",
					ChatColor.GRAY+"/m g:<NomGroupe> <Message>"+ChatColor.GOLD+" pour envoyer un message dans une discussion avec plusieurs joueurs",
					ChatColor.GOLD+"----------------------------------------------",
			};
			
			sender.sendMessage(m);
		}
	}
	
	
	
	private void onCommandSend(CommandSender sender, String target, String message) {
		
		if (sender instanceof Player && OnlinePlayerManager.get((Player) sender).getEssentialsUser().isMuted()) {
			sender.sendMessage(ChatColor.RED+"Vous ne pouvez pas envoyer de message quand vous êtes muet");
			return;
		}
		
		try {
			plugin.privateMessagesManager.inputMessage(new MessageSender(sender), target, message);
		} catch (MessageSendingException e) {
			sender.sendMessage(ChatColor.RED+e.getMessage());
		}
		
	}
	
	
	
	
	private void onCommandDiscussionsList(Player player, int number) {
		List<PrivateDiscussion> lastDiscussions = plugin.privateMessagesManager.getLastDiscussions(player, number);
		

		player.sendMessage(ChatColor.GOLD+"--- Messagerie privée - Liste des conversations ---");
		
		for (PrivateDiscussion pvd : lastDiscussions) {
			String name = (pvd.name == null) ?
					new MessageSender(pvd.name).getDisplayName() :
					(pvd.name.toLowerCase().startsWith("g:")) ?
							ChatColor.GOLD+pvd.name :
							new MessageSender(pvd.name).getDisplayName();
			
			String line = name+ChatColor.RESET;
			
			int nbM = pvd.nbUnread;
			
			if (nbM > 0)
				line += ChatColor.GRAY+" - "+ChatColor.LIGHT_PURPLE+ChatColor.BOLD+nbM+" message"+((nbM > 1)?"s":"")+" non lu"+((nbM > 1)?"s":"");
			

			DateFormat date = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.FRANCE);
			DateFormat hour = DateFormat.getTimeInstance(DateFormat.MEDIUM, Locale.FRANCE);
			
			String affDate = date.format(pvd.timeLastMessage) + " à " + hour.format(pvd.timeLastMessage);
			
			line += ChatColor.GRAY+" - "+affDate;
			
			String messageExtract = pvd.lastMessage;
			
			if (messageExtract.length()>50)
				messageExtract = messageExtract.subSequence(0, 47)+"...";
			
			
			new FancyMessage("").then(line)
					.color(ChatColor.GRAY)
					.command("/m d:"+((pvd.name == null)?"null":pvd.name))
					.tooltip(ChatColor.GRAY+"Cliquez pour lire les derniers messages",
							ChatColor.GRAY+"Dernier message de "+ChatColor.RESET+pvd.lastMessageSource.getDisplayName(),
							ChatColor.WHITE.toString()+ChatColor.ITALIC+messageExtract
							)
				.send(player);
			
		}

		player.sendMessage(ChatColor.GOLD+"Cliquez sur une conversation pour voir les messages");
		player.sendMessage(ChatColor.GOLD+"----------------------------------------------");
		
	}
	
	
	
	
	private void onCommandDiscussionDisplay(Player player, String discussionName, int nbMin) {

		player.sendMessage(ChatColor.GOLD+"--------------- Messagerie privée ---------------");
		plugin.privateMessagesManager.displayLastDiscussionMessage(player, discussionName, nbMin, 3);
		player.sendMessage(ChatColor.GOLD+"----------------------------------------------");
		
	}
	
	

}
