package net.mc_pandacraft.java.plugin.pandacraftutils.modules.player_control;

import java.sql.Date;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.function.Predicate;

import org.apache.commons.lang.WordUtils;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.Sign;
import org.bukkit.craftbukkit.v1_7_R1.block.CraftSign;

import com.earth2me.essentials.User;
import com.earth2me.essentials.UserMap;

import net.mc_pandacraft.java.plugin.pandacraftutils.PandacraftUtils;
import net.mc_pandacraft.java.plugin.pandacraftutils.config.ConfigManager;
import net.mc_pandacraft.java.plugin.pandacraftutils.plugin_interface.EssentialsInterface;

public class JailsManager {
	private PandacraftUtils plugin = PandacraftUtils.getInstance();
	
	private List<String> jails;
	private Map<String, JailedPlayer> jailedPlayers;

	
	
	public JailsManager() {
		
		// initialisation retardée (besoin des données d'un autre plugin)
		plugin.getServer().getScheduler().runTaskLater(plugin, new Runnable() {
			@Override
			public void run() {
				updateDataFromEssentials();
			}
		}, 15);
	}
	
	
	
	
	
	
	
	
	
	/**
	 * En principe, cette méthone ne doit-être appelée qu'une seule fois, après le chargement
	 * du plugin Essentials
	 */
	private void updateDataFromEssentials() {
		plugin.getLogger().info("[JailsManager] chargement des données de la prison depuis Essentials");
		try {
			jails = new ArrayList<String>(EssentialsInterface.getPlugin().getJails().getList());
			
			// retirer les cellules qui n'ont pas la forme "cX" avec X un nombre
			jails.removeIf(new Predicate<String>() {
				@Override public boolean test(String t) {
					return t == null || !t.matches("c[0-9]+");
				}
			});
			
			// on trie les cellules, pour qu'elle se remplissent toujours dans le même ordre
			Collections.sort(jails, new Comparator<String>() {
				@Override public int compare(String o1, String o2) {
					try {
						int i1 = Integer.parseInt(o1.substring(1));
						int i2 = Integer.parseInt(o2.substring(1));
						if (i1 < i2) return -1;
						if (i1 > i2) return 1;
						return 0;
					} catch (NumberFormatException e) { return 0; }
				}
			});
			
			UserMap um = EssentialsInterface.getPlugin().getUserMap();
			
			Set<String> users = um.getAllUniqueUsers();
			
			jailedPlayers = new HashMap<String, JailedPlayer>();
			
			// on parcours les joueurs et on ajoute ceux qui sont emprisonnés, dans les données
			for (String user : users) {
				User u = um.getUser(user);
				if (u.isJailed() && (u.getJailTimeout() == 0 || u.getJailTimeout() >= System.currentTimeMillis())) {
					String jail = u.getJail();
					if (jails.contains(jail)) {
						jailedPlayers.put(jail, new JailedPlayer(user, jail, null, u.getJailTimeout()));
						
						
					}
				}
			}
			
			updateJailsSign();
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		plugin.getLogger().info("[JailsManager] chargement terminé");
	}
	
	
	
	
	
	
	/**
	 * Emprisonne le joueur passé en paramètre. Les données Essentials sont mise
	 * à jour à l'appel de cette méthode.
	 * @param duree en seconde
	 */
	public boolean addJailedPlayer(User player, String raison, long duree) {
		if (player.isJailed() && (player.getJailTimeout() == 0 || player.getJailTimeout() >= System.currentTimeMillis()))
			return false; // le joueur est déjà en prison
		
		String jail = getFirstAvailableJail();
		
		if (jail == null)
			return false; // aucune cellule de libre
		
		player.setJailed(true);
		player.setJail(jail);
		player.setJailTimeout(System.currentTimeMillis()+duree*1000);
		
		if (player.isOnline()) {
			try {
				player.teleport(EssentialsInterface.getPlugin().getJails().getJail(jail));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		jailedPlayers.put(jail, new JailedPlayer(player.getName(), jail, raison, System.currentTimeMillis()+duree*1000));
		
		
		updateJailsSign();
		
		return true;
		
	}
	
	
	
	public boolean removeJailedPlayer(User player) {
		if (!player.isJailed() || player.getJailTimeout() == 0 || player.getJailTimeout() < System.currentTimeMillis())
			return false; // le joueur n'est pas en prison
		
		String jail = player.getJail();
		
		JailedPlayer jp = jailedPlayers.get(jail);
		
		if (jp == null || !jp.playerName.equalsIgnoreCase(player.getName()))
			return false; // incohérence dans les données
		
		player.setJailed(false);
		
		jailedPlayers.remove(jail);
		
		updateJailsSign();
		
		return true;
		
	}
	
	
	
	
	private String getFirstAvailableJail() {
		
		// supprime d'abord les joueurs dont leur emprisonnement est expiré
		for (String jail : jailedPlayers.keySet().toArray(new String[jailedPlayers.size()])) {
			if (jailedPlayers.get(jail).timeout < System.currentTimeMillis())
				jailedPlayers.remove(jail);
		}
		
		
		
		for (String j : jails) {
			if (jailedPlayers.containsKey(j))
				continue;
			return j;
		}
		return null;
	}
	
	
	
	
	
	private void updateJailsSign() {
		/*
		 * Disposition des panneaux :
		 * 
		 * <Pseudo>				Raison :
		 * libéré(e) le			<Raison>
		 * <Date>
		 * <Heure>
		 * 
		 * La raison peut prendre plusieurs panneaux
		 */
		
		
		try {

			int wallWidth = ConfigManager.getInstance().defaultConfig.modo_jailWalls_width;
			int wallHeight = ConfigManager.getInstance().defaultConfig.modo_jailWalls_height;
			BlockFace wallDir = ConfigManager.getInstance().defaultConfig.modo_jailWalls_direction;
			
		
			for (String jail : jails) {
				
				Location jailLoc = EssentialsInterface.getPlugin().getJails().getJail(jail);
				
				Block topLeftWallBloc = jailLoc.getBlock()
						.getRelative(wallDir)
						.getRelative(BlockFace.UP, wallHeight-1)
						.getRelative(wallDir.getModZ(), 0, -wallDir.getModX());
				
				// liste de tout les panneaux de la cellulre
				Block[] blocks = new Block[wallWidth*wallHeight];
				int n = 0;
				for (int i=0; i<wallHeight; i++)
					for (int j=0; j<wallWidth; j++) {
						Block b = topLeftWallBloc;
						if (i != 0)
							b = b.getRelative(BlockFace.DOWN, i);
						if (j != 0)
							b = b.getRelative(-wallDir.getModZ()*j, 0, wallDir.getModX()*j);
						
						blocks[n] = b;
						n++;
					}
				
				// joueur emprisonné (si il existe)
				JailedPlayer jp = jailedPlayers.get(jail);
				if (jp == null || jp.timeout < System.currentTimeMillis()) {
					// pas de joueur emprisonné
					
					for (Block b : blocks)
						b.setType(Material.AIR);
					
				}
				else {
					String player = jp.playerName;
					player = (player.length() > 15) ? player.substring(0, 15) : player;
					
					String raison = jp.reason;
					if (raison != null)
						raison = WordUtils.wrap(raison, 15, "\n", true);
					
					Date d = new Date(jp.timeout);
					DateFormat date = DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.FRANCE);
					DateFormat hour = DateFormat.getTimeInstance(DateFormat.MEDIUM, Locale.FRANCE);
					
					String affDate = date.format(d) + " à " + hour.format(d);
					affDate = WordUtils.wrap(affDate, 15, "\n", true);
					String[] affDateLines = affDate.split("\n");
					
					// premier panneau (<pseudo> libéré le <date> à <heure>)
					if (blocks.length >= 1) {
						String l1 = player;
						String l2 = "libéré(e) le";
						String l3 = (affDateLines.length>0)?affDateLines[0]:"";
						String l4 = (affDateLines.length>1)?affDateLines[1]:"";
						setWallSign(blocks[0], l1, l2, l3, l4, wallDir);
					}
					
					if (raison != null) {
						// on ne met à jour que si la raison est précisé (effacé après redémarrage, mais
						// il ne faut pas l'effacer des pannaux si le joueur est toujours en prison)
						String[] raisonLines = raison.split("\n");
						
						// debug
						plugin.getLogger().info(Arrays.asList(raisonLines).toString());
						
						int raisonI = 0;
						
						for (int i=1; i<blocks.length; i++) {
							
							setWallSign(blocks[i],
									(raisonI<raisonLines.length)?raisonLines[raisonI]:"",
									(raisonI+1<raisonLines.length)?raisonLines[raisonI+1]:"",
									(raisonI+2<raisonLines.length)?raisonLines[raisonI+2]:"",
									(raisonI+3<raisonLines.length)?raisonLines[raisonI+3]:"",
									wallDir);
							
							
							raisonI+=4;
						}
						
						
						
					}
					
					
					
				}
				
				
				
				
				
			}
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		
	}
	
	
	private void setWallSign(Block b, String l1, String l2, String l3, String l4, BlockFace wallDir) {
		b.setType(Material.WALL_SIGN);
		
		Sign s = new CraftSign(b);
		
		org.bukkit.material.Sign matS = new org.bukkit.material.Sign(Material.WALL_SIGN);
		matS.setFacingDirection(wallDir.getOppositeFace());
		
		s.setData(matS);
		s.setLine(0, l1);
		s.setLine(1, l2);
		s.setLine(2, l3);
		s.setLine(3, l4);
		
		s.update();
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	public class JailedPlayer {
		public final String playerName;
		public final String jail;
		/**
		 * Si cette valeur est nulle, on ne met pas à jour les panneaux
		 */
		public final String reason;
		public final long timeout;
		
		public JailedPlayer(String n, String j, String r, long t) {
			playerName = n;
			jail = j;
			reason = r;
			timeout = t;
		}
	}
	
	
	
	
}
