package net.mc_pandacraft.java.plugin.pandacraftutils.modules.staff;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import net.mc_pandacraft.java.plugin.pandacraftutils.PandacraftUtils;
import net.mc_pandacraft.java.plugin.pandacraftutils.data_model.ORM;
import net.mc_pandacraft.java.plugin.pandacraftutils.data_model.StaffTicketElement;
import net.mc_pandacraft.java.plugin.pandacraftutils.data_model.StaffTicketTable;
import net.mc_pandacraft.java.plugin.pandacraftutils.players.OnlinePlayerManager;
import net.mc_pandacraft.java.util.ScoreBoardUtil;
import net.mc_pandacraft.java.util.TimeUtil;

import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.scoreboard.Scoreboard;

public class StaffQueueManager {
	
	private PandacraftUtils plugin = PandacraftUtils.getInstance();
	
	
	private Queue<WaitingPlayer> playerQueue = new LinkedList<WaitingPlayer>();
	
	private Scoreboard scoreboardPlayerQueue = Bukkit.getScoreboardManager().getNewScoreboard();
	
	
	
	public StaffQueueManager() {
		
		try {
			StaffTicketTable dbTable = (StaffTicketTable) ORM.getTable("staff_ticket");
			
			List<StaffTicketElement> elmts = dbTable.getAll(null, "creationTime ASC, id ASC", null, null);
			
			for (StaffTicketElement el : elmts)
				playerQueue.add(new WaitingPlayer(el));
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		updateScoreBoardWaitingList();
	}
	
	
	
	
	// le joueur ajouté doit-être en ligne
	public void addWaitingPlayer(Player p, String m) {
		if (getWaitingPlayer(p) != null) return; // le joueur est déjà dans la file
		playerQueue.add(new WaitingPlayer(p.getName(), m, p.getLocation()));
		updateScoreBoardWaitingList();
	}
	
	
	
	
	
	public WaitingPlayer getWaitingPlayer(String p) {
		for (WaitingPlayer wp : playerQueue)
		{
			if (wp != null && wp.getPlayerName().equalsIgnoreCase(p))
				return wp;
		}
		return null;
	}
	
	
	public WaitingPlayer getWaitingPlayer(OfflinePlayer p) {
		return getWaitingPlayer(p.getName());
	}
	
	
	
	
	public int getPlayerPosition(OfflinePlayer p) {
		int i = 0;
		for (WaitingPlayer wp : playerQueue)
		{
			i++;
			if (wp != null && wp.getPlayerName().equals(p.getName()))
				return i;
		}
		return -1;
	}
	

	
	public void removePlayer(String p) {
		WaitingPlayer wp = getWaitingPlayer(p);
		if (wp != null)
		{
			playerQueue.remove(wp);
			wp.deleteFromDB();
			updateScoreBoardWaitingList();
		}
	}
	public void removePlayer(OfflinePlayer p) {
		removePlayer(p.getName());
	}
	
	
	
	
	
	public WaitingPlayer getSelectedWaitingPlayer(Player staffPlayer) {
		String staffName = staffPlayer.getName();
		for (WaitingPlayer wp : playerQueue)
		{
			if (wp != null && wp.getStaffPlayerName() != null && wp.getStaffPlayerName().equals(staffName))
				return wp;
		}
		return null;
	}
	
	
	
	
	
	public void updateScoreBoardWaitingList() {
		
		// contient les joueurs qui sont associés à un membre du staff
		ArrayList<String> selectedPlayers = new ArrayList<String>();
		
		// contient les joueurs qui ne sont pas encore associés à un membre du staff connecté
		ArrayList<String> newPlayers = new ArrayList<String>();
		
		int i = 1;
		for (WaitingPlayer wp : playerQueue) {
			String line = i+"."+wp.getPlayerDisplayName();
			if (wp.getStaffPlayerName() == null)
				newPlayers.add(line);
			else
				selectedPlayers.add(line);
			i++;
		}
		
		
		// contients les lignes qui vont être affichés dans la sidebar
		ArrayList<String> lines = new ArrayList<String>();
		
		if (!selectedPlayers.isEmpty()) {
			lines.add(ChatColor.GREEN+"En traitement:");
			lines.addAll(selectedPlayers);
		}
		if (!newPlayers.isEmpty()) {
			lines.add(ChatColor.GOLD+"En attente :");
			lines.addAll(newPlayers);
		}
		
		if (!lines.isEmpty()) {
			lines.add("");
			lines.add("Commande :");
			lines.add(ChatColor.GRAY+"   /staff");
		}
		
		ScoreBoardUtil.updateScoreboardSidebar(scoreboardPlayerQueue, ChatColor.GOLD+"File d'attente staff", lines);
	}
	
	
	
	

	public void showWaintingList(CommandSender sender) {
		
		
		sender.sendMessage(ChatColor.BOLD+"--- File d'attente des joueurs pour le staff ---");

		int i=1;
		for (WaitingPlayer wp : playerQueue) {
			
			String staffPlayer = wp.getStaffPlayerName();
			if (staffPlayer != null)
				staffPlayer = " ("+staffPlayer+ChatColor.RESET+")";
			else
				staffPlayer = "";
			
			sender.sendMessage(i+"."+staffPlayer+" "+wp.getPlayerDisplayName()+ChatColor.RESET+" : "+wp.getMessage()+" - "
					+ChatColor.ITALIC+"Il y a "+TimeUtil.durationToString((new Date()).getTime() - wp.getDate().getTime()));
			i++;
		}
		sender.sendMessage(ChatColor.BOLD+"-------------------------------------------");
		
		
	}
	
	
	
	
	
	public List<String> getWaitingPlayerNames() {
		List<String> l = new ArrayList<String>();
		
		for (WaitingPlayer wp : playerQueue) {
			l.add(wp.getPlayerName());
		}
		return l;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public void onPlayerJoin(PlayerJoinEvent event) {
		Player p = event.getPlayer();
		if (OnlinePlayerManager.get(p).isInStaff())
			p.setScoreboard(scoreboardPlayerQueue);
		plugin.getServer().getScheduler().runTaskLater(plugin, new Runnable() {
			@Override public void run() { updateScoreBoardWaitingList(); }
		}, 2L);
		
	}
	
	
	public void onPlayerQuit(PlayerQuitEvent event) {
		
		// on vérifie si un membre du staff se déconnecte
		for (WaitingPlayer wp : playerQueue) {
			if (wp.getStaffPlayerName() != null && wp.getStaffPlayerName().equals(event.getPlayer().getName()))
				wp.setStaffPlayer(null);
		}
		
		plugin.getServer().getScheduler().runTaskLater(plugin, new Runnable() {
			@Override public void run() { updateScoreBoardWaitingList(); }
		}, 2L);
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public class WaitingPlayer {
		
		private String player;
		private String message;
		private Date creationDate;
		private Location location; // localisation du joueur au moment où il fait /staff
		private String staffPlayer = null;
		
		private int dbId;
		

		public WaitingPlayer(String p, String m, Location l) {
			
			player = p;
			message = m;
			creationDate = new Date();
			location = l;
			
			StaffTicketElement dbEl = new StaffTicketElement(p, m, creationDate.getTime(), l);
			dbEl.save();
			
			dbId = dbEl.getId();
		}
		
		private WaitingPlayer(StaffTicketElement dbEl) {
			
			player = dbEl.getPlayerName();
			message = dbEl.getMessage();
			creationDate = new Date(dbEl.getCreationTime());
			location = dbEl.getLocation();
			
			staffPlayer = dbEl.getStaffPlayer();
			
			dbId = dbEl.getId();
		}
		
		/**
		 * 
		 * @return le pseudo du joueur
		 */
		public String getPlayerName() { return player; }
		
		/**
		 * 
		 * @return le pseudo du joueur, avec la coloration si il est connecté
		 */
		public String getPlayerDisplayName() {
			Player p = plugin.getServer().getPlayerExact(player);
			if (p == null || !p.isOnline())
				return player;
			else
				return p.getPlayerListName();
		}
		
		public String getMessage() { return message; }
		
		public void setMessage(String m) {
			if (m == null) return;
			message = m;
			
			try {
				StaffTicketElement el = ((StaffTicketTable)ORM.getTable("staff_ticket")).get(dbId);
				
				el.setMessage(message);
				el.save();
			} catch (Exception e) {
				plugin.getLogger().severe("Impossible de récupérer l'élément en base de donnée : id="+dbId);
				e.printStackTrace();
			}
			
		}
		
		public Date getDate() { return creationDate; }
		
		public Location getLocation() { return location; }
		
		public void setStaffPlayer(Player p) {
			if (p == null) staffPlayer = null;
			else staffPlayer = p.getName();
			
			try {
				StaffTicketElement el = ((StaffTicketTable)ORM.getTable("staff_ticket")).get(dbId);
				
				el.setStaffPlayer(staffPlayer);
				el.save();
			} catch (Exception e) {
				plugin.getLogger().severe("Impossible de récupérer l'élément en base de donnée : id="+dbId);
				e.printStackTrace();
			}
			
			updateScoreBoardWaitingList();
		}
		public String getStaffPlayerName() { return staffPlayer; }
		
		public void deleteFromDB() {
			try {
				((StaffTicketTable)ORM.getTable("staff_ticket")).get(dbId).delete();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
}
